'''
Created on Feb 10, 2016

@author: jivan
'''
import base64
import cPickle
from cStringIO import StringIO
from datetime import date
import uuid

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail.message import EmailMultiAlternatives
from django.core.urlresolvers import reverse
from django.db import models
from django.shortcuts import render
from django.template.loader import render_to_string


# Collect the number of days between confirmation emails from settings.
#    Default to 3 if it's not provided.
EMAIL_CONFIRMATION_WAIT = settings.__dict__.get('EMAIL_CONFIRMATION_WAIT', 3)
EMAIL_CONFIRMATION_TEMPLATE_TXT = \
    getattr(settings, 'EMAIL_CONFIRMATION_TEMPLATE_TXT', 'confirmed_email/confirmation_email.txt')
EMAIL_CONFIRMATION_TEMPLATE_HTML = \
    getattr(settings, 'EMAIL_CONFIRMATION_TEMPLATE_HTML', 'confirmed_email/confirmation_email.html')

class AddressConfirmation(models.Model):
    ''' Stores addresses with their confirmation status. '''
    address = models.EmailField()
    # unique string used as part of the confirmation link.
    uuid = models.CharField(max_length=32, default=uuid.uuid1)
    # This is None for unconfirmed addresses or the timestamp when the user clicked
    #    the confirmation link.
    confirmation_timestamp = models.DateTimeField(null=True)
    last_request_date = models.DateField(null=True)
    request_count = models.IntegerField(default=0)

    def send_confirmation_request(self, from_address):
        ''' | *brief*: Sends an email requesting that *from_address* be confirmed.
            | *author*: Jivan
            | *created*: 2016-03-02
            Sends an email if one hasn't been sent in more than EMAIL_CONFIRMATION_WAIT days.
            Like EmailMessage, returns 0 for an email sending failure, 1 for sending success.
            A value of 1 is also returned for a message skipped due to a previous message
            sent within EMAIL_CONFIRMATION_WAIT days.
        '''
        if self.last_request_date is None:
            days_since_request = None
        else:
            delta = date.today() - self.last_request_date
            days_since_request = delta.days

        ret = 1
        if days_since_request is None or days_since_request > EMAIL_CONFIRMATION_WAIT:
            current_domain = Site.objects.get_current().domain

            confirmation_location = \
                reverse('confirmed-email-confirmation-url', kwargs={'uuid': self.uuid})
            confirmation_link = 'http://' + current_domain + confirmation_location
            message_context = {'confirmation_link': confirmation_link}
            message_body_text = render_to_string(EMAIL_CONFIRMATION_TEMPLATE_TXT, message_context)
            message_body_html = render_to_string(EMAIL_CONFIRMATION_TEMPLATE_HTML, message_context)

            # Confirmation Email
            em = EmailMultiAlternatives(subject='Please confirm your email address',
                              body=message_body_text,
                              from_email=from_address,
                              to=[self.address])
            em.attach_alternative(message_body_html, "text/html")

            ret = em.send()
            if ret:
                self.last_request_date = date.today()
                self.save()

        return ret


class QueuedEmailMessage(models.Model):
    ''' Stores unsent email messages while waiting for confirmation.'''
    address_confirmation = models.ForeignKey(AddressConfirmation)
    # Date when the message was queued while awaiting confirmation.
    date = models.DateField(auto_now=True)
    # ConfirmedEmailMessage instance serialized with json-pickle.
    _email_contents = models.TextField(db_column='email_contents', blank=True)

    @property
    def email_contents(self):
        # --- Decode & unpickle the message
        pickle_input = base64.decodestring(self._email_contents)
        # In-memory pickle input
        pi = StringIO(pickle_input)
        message = cPickle.load(pi)

        return message

    @email_contents.setter
    def email_contents(self, email_message):
        # --- Pickle & encode the message for storage in a TextField.
        # In-memory pickle output.
        po = StringIO()
        cPickle.dump(email_message, po)
        pickle_output = po.getvalue()
        po.close()

        self._email_contents = base64.encodestring(pickle_output)

    def send(self):
        res = self.email_contents.send()
        return res



'''
Created on Feb 10, 2016

@author: jivan
'''
import copy
from django.core.mail.message import EmailMultiAlternatives
from confirmed_email.models import AddressConfirmation, QueuedEmailMessage
import logging
logger = logging.getLogger(__name__)

def send_mail_confirmed(subject, message, from_email, recipient_list,
        fail_silently=False, auth_user=None, auth_password=None,
        connection=None, html_message=None
    ):
    from django.core.mail import get_connection
    connection = connection or get_connection(username=auth_user,
                                    password=auth_password,
                                    fail_silently=fail_silently)
    return ConfirmedEmailMessage(subject, message, from_email, recipient_list,
                        connection=connection).send()

class ConfirmedEmailMessage(EmailMultiAlternatives):
    def send(self, *args, **kwargs):
        ''' | *brief*: Sends message, preceding it with a confirmation email or
            |    unconfirmed addresses.
            | *author*: Jivan
            | *created*: 2016-02-23
            In the parent's send() the return value is the number of email messages
            processed which would be 0 (for failure) or 1 (for success).

            This class returns a dictionary with all recipient
            addresses as keys and the text 'sent', 'queued', or
            'failed' as the value
            for each key.
        '''
        destination_count = len(self.recipients())
        confirmed = AddressConfirmation.objects.filter(
                        address__in=self.recipients(),
                        confirmation_timestamp__isnull=False)
        confirmed_addresses = [ c.address for c in confirmed ]
        confirmed_count = len(confirmed_addresses)

        send_results = {}
        # If all the destination addresses are confirmed, send as-is.
        if destination_count == confirmed_count:
            if super(ConfirmedEmailMessage, self).send():
                send_results.update({ ca: 'sent' for ca in confirmed_addresses })
        else:
            # If any of the destination addresses are unconfirmed, send to
            #    each individually.
            for recipient in self.recipients():
                cem = ConfirmedEmailMessage(
                    self.subject, self.body, self.from_email,
                    connection=self.connection, attachments=self.attachments,
                    headers=self.extra_headers, alternatives=self.alternatives
                )
                cem.to = [recipient]
                cem.cc = []
                cem.bcc = []
                if recipient in confirmed_addresses:
                    single_send_result = cem.send()
                    send_results.update(single_send_result)
                else:
                    # sending_results is the number of messages sent.
                    sending_result = cem._send_unconfirmed()
                    if sending_result:
                        send_results.update({recipient: 'queued'})
                    else:
                        send_results.update({recipient: 'failed'})

        return send_results

    def _send_unconfirmed(self):
        ''' | *brief*: Queues this message for later delivery and sends a confirmation
            |     message to the recipient.
            | *note*: Only single-recipient messages are allowed.
            *returns*: 0 if sending confirmation message failed, 1 if sending
                confirmation message succeeded.
        '''
        if len(self.recipients()) > 1:
            msg = '_send_unconfirmed() should not be used directly.  Use send().'
            logger.error(msg)
        address = self.recipients()[0]

        # Add address to EmailAddresses
        ac = AddressConfirmation.objects.get_or_create(address=address)[0]
        # Queue message
        qem = QueuedEmailMessage.objects.create(address_confirmation=ac, email_contents=self)
        if not qem:
            logger.error('Unable to create QueuedEmailMessage to {}'.format(address))
        # Send confirmation email
        confirmation_message_error = ac.send_confirmation_request(from_address=self.from_email)
        return confirmation_message_error

    def __eq__(self, other):
        # Assume equal
        equal = True

        # If the number of attachments differs, not equal
        if len(self.attachments) != len(other.attachments):
            equal = False

        # If the properties differ, not equal
        if equal and (self.__dict__ != other.__dict__):
            equal = False

        # If the attachments differ, not equal
        if equal:
            for at1, at2 in zip(self.attachments, other.attachments):
                if at1 != at2:
                    equal = False
                    break
        return equal

    def __unicode__(self):
        u = u''
        for key, val in self.__dict__.items():
            u += '{}: {}\n'.format(key, val)
        return u

    def __getstate__(self):
        d = dict(self.__dict__)
        del d['connection']
        return d

    def __setstate__(self, d):
        self.__dict__.update(d)
        self.connection = False



'''
Created on Feb 10, 2016

@author: jivan
'''
from datetime import datetime
import logging

from django.shortcuts import render
from django.template.context import RequestContext
from django.views.generic import TemplateView

from confirmed_email.models import AddressConfirmation, QueuedEmailMessage
from django.conf import settings

ADDRESS_CONFIRMED_TEMPLATE = \
    getattr(settings, 'ADDRESS_CONFIRMED_TEMPLATE', 'confirmed_email/address_confirmed.html')


logger = logging.getLogger(__name__)


class HandleConfirmationClick(TemplateView):
    template_name = ADDRESS_CONFIRMED_TEMPLATE
    def get(self, request, uuid):
        # Mark the email associated with uuid as confirmed.
        ac = AddressConfirmation.objects.get(uuid=uuid)
        ac.confirmation_timestamp = datetime.now()
        ac.save()
        # Send any emails to this address which are waiting.
        send_queued_emails(ac)

        # Provide a page to thank the user for confirming.
        rc = RequestContext(request)
        rc.update({'email_address': ac.address})
        resp = render(request, self.template_name, rc)
        return resp


def send_queued_emails(address_confirmation):
    # Send queued emails for *address_confirmation*, provided the address has
    #    indeed been confirmed.
    # Afterwards send queued emails for any remaining to confirmed addresses.
    if address_confirmation.confirmation_timestamp:
        # queued emails
        qems = QueuedEmailMessage.objects.filter(address_confirmation=address_confirmation)
        for qem in qems:
            qem.send()
            qem.delete()
    else:
        logger.warn(
            'Attempt to send queued emails for unconfirmed address: {}'.format(address_confirmation)
        )

    # Check for and send queued emails for any destination addresses that have been confirmed.
    qems = QueuedEmailMessage.objects.filter(address_confirmation__confirmation_timestamp__isnull=False)
    for qem in qems:
        qem.send()
        qem.delete()



import os
SECRET_KEY = 'NOT_REALLY_IMPORTANT_FOR_TESTING',
DEBUG = True,

DATABASES = {
    'default': {
        'NAME': 'confirmed-email.sqlite3',
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

ROOT_URLCONF = 'confirmed_email.urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.admin',
    'confirmed_email',
)



#!/usr/bin/env python
import os
import sys

package_parent_dir = os.path.normpath(
    os.path.join(os.path.abspath(__file__), os.path.pardir, os.path.pardir)
)
sys.path = [package_parent_dir] + sys.path
os.environ['DJANGO_SETTINGS_MODULE'] = 'confirmed_email.settings_test'

from django.conf import settings
from django.test import utils
from django.test.utils import get_runner


if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "confirmed_email.settings")
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)



from setuptools import setup
import os

package_path = os.path.normpath(os.path.join(os.path.abspath(__file__), os.path.pardir))
os.chdir(package_path)

with open('README.rst') as rm:
    README = rm.read()

confirmed_email_data = [
    'templates/confirmed_email/address_confirmed.html',
    'templates/confirmed_email/confirmation_email.txt',
    'templates/confirmed_email/confirmation_required.html'
]

setup(
    author="Jivan Amara",
    author_email='Development@JivanAmara.net',
    url='https://github.com/JivanAmara/confirmed-email',
    name='django-confirmed-email',
    version='0.0.4',
    packages=['confirmed_email'],
    package_data={'confirmed_email': confirmed_email_data},
    description='Provides an email sender that automatically confirms addresses.',
    long_description=README,
    classifiers=[
        'Programming Language :: Python',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
    ]
)



'''
Created on Feb 11, 2016

@author: jivan
'''
from django.conf.urls import url, patterns
from confirmed_email.views import HandleConfirmationClick

urlpatterns = patterns('',
    url(r'^confirm-email-address/(?P<uuid>.*?)$', HandleConfirmationClick.as_view(),
        name='confirmed-email-confirmation-url'
    ),
)
